@@ -1,5 +1,6 @@
 use regex::Regex;
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
+use std::fmt;
 
 use crate::parsers;
 use crate::parsers::parser_line::tokens_to_redirections;
@@ -7,7 +8,115 @@ use crate::shell;
 use crate::libs;
 use crate::tools;
 
-pub const STOPPED: i32 = 148;
+#[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
+pub struct WaitStatus(i32, i32, i32);
+
+impl WaitStatus {
+    pub fn from_exited(pid: i32, status: i32) -> Self {
+        WaitStatus(pid, 0, status)
+    }
+
+    pub fn from_signaled(pid: i32, sig: i32) -> Self {
+        WaitStatus(pid, 1, sig)
+    }
+
+    pub fn from_stopped(pid: i32, sig: i32) -> Self {
+        WaitStatus(pid, 2, sig)
+    }
+
+    pub fn from_continuted(pid: i32) -> Self {
+        WaitStatus(pid, 3, 0)
+    }
+
+    pub fn from_others() -> Self {
+        WaitStatus(0, 9, 9)
+    }
+
+    pub fn from_error(errno: i32) -> Self {
+        WaitStatus(0, 255, errno)
+    }
+
+    pub fn empty() -> Self {
+        WaitStatus(0, 0, 0)
+    }
+
+    pub fn is_error(&self) -> bool {
+        self.1 == 255
+    }
+
+    pub fn is_others(&self) -> bool {
+        self.1 == 9
+    }
+
+    pub fn is_signaled(&self) -> bool {
+        self.1 == 1
+    }
+
+    pub fn get_errno(&self) -> nix::Error {
+        nix::Error::from_i32(self.2)
+    }
+
+    pub fn is_exited(&self) -> bool {
+        self.0 != 0 && self.1 == 0
+    }
+
+    pub fn is_stopped(&self) -> bool {
+        self.1 == 2
+    }
+
+    pub fn is_continued(&self) -> bool {
+        self.1 == 3
+    }
+
+    pub fn get_pid(&self) -> i32 {
+        self.0
+    }
+
+    fn _get_signaled_status(&self) -> i32 {
+        self.2 + 128
+    }
+
+    pub fn get_signal(&self) -> i32 {
+        self.2
+    }
+
+    pub fn get_name(&self) -> String {
+        if self.is_exited() {
+            "Exited".to_string()
+        } else if self.is_stopped() {
+            "Stopped".to_string()
+        } else if self.is_continued() {
+            "Continued".to_string()
+        } else if self.is_signaled() {
+            "Signaled".to_string()
+        } else if self.is_others() {
+            "Others".to_string()
+        } else if self.is_error() {
+            "Error".to_string()
+        } else {
+            format!("unknown: {}", self.2)
+        }
+    }
+
+    pub fn get_status(&self) -> i32 {
+        if self.is_exited() {
+            self.2
+        } else {
+            self._get_signaled_status()
+        }
+    }
+}
+
+impl fmt::Debug for WaitStatus {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let mut formatter = f.debug_struct("WaitStatus");
+        formatter.field("pid", &self.0);
+        let name = self.get_name();
+        formatter.field("name", &name);
+        formatter.field("ext", &self.2);
+        formatter.finish()
+    }
+}
 
 pub type Token = (String, String);
 pub type Tokens = Vec<Token>;
@@ -116,8 +225,24 @@ pub struct Job {
     pub id: i32,
     pub gid: i32,
     pub pids: Vec<i32>,
+    pub pids_stopped: HashSet<i32>,
     pub status: String,
-    pub report: bool,
+    pub is_bg: bool,
+}
+
+impl Job {
+    pub fn all_members_stopped(&self) -> bool {
+        for pid in &self.pids {
+            if !self.pids_stopped.contains(&pid) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    pub fn all_members_running(&self) -> bool {
+        self.pids_stopped.is_empty()
+    }
 }
 
 #[derive(Clone, Debug, Default)]
